@@ -2,6 +2,14 @@
 `binteger` is a toolkit for manipulating integers in their binary
 (fixed-width) representation (big endian - most significant bits go first).
 
+**Documentation**: https://binteger.readthedocs.io/
+
+**Code**: https://github.com/hellman/binteger
+
+**Installation**::
+
+    pip install binteger
+
 This module is quite similar to `bitstring` (at PyPI):
 https://github.com/scott-griffiths/bitstring
 which is pretty awesome.
@@ -22,6 +30,7 @@ True
 .. warning:: `binteger` is optimized for *convenience of use*,
               rather than for performance.
 """
+from random import randrange
 from functools import reduce
 
 
@@ -199,6 +208,32 @@ class Bin:
             raise ValueError("integer out of range")
         return cls(1 << (n - 1 - i % n), n)
 
+    @classmethod
+    def random(cls, n, nonzero=False):
+        r"""Random `n`-bit Bin.
+
+        Parameters
+        ----------
+        n : int
+            The width.
+        nonzero : bool = False
+            Force nonzero integer?
+
+        Examples
+        --------
+        >>> Bin.random(0)
+        Bin(0b0, n=0)
+        >>> Bin.random(10)  # doctest: +SKIP
+        Bin(0b0110110011, n=10)
+        >>> Bin.random(10)  # doctest: +SKIP
+        Bin(0b1010000111, n=10)
+        >>> all(Bin.random(3) for _ in range(1000))
+        False
+        >>> all(Bin.random(3, nonzero=True) for _ in range(1000))
+        True
+        """
+        return cls(randrange(1 if nonzero else 0, 2**n), n)
+
     @property
     def mask(self):
         """Bin: Binary mask for the full width."""
@@ -435,19 +470,35 @@ class Bin:
             raise ValueError("Can not compare Bin's with different n")
         return self.int == other.int
 
-    # disabled because ambiguous (with prec)
+    def __lt__(self, other):
+        """
+        >>> Bin(3, 5) < Bin(4, 5)
+        True
+        >>> Bin(3, 5) < Bin(3, 5)
+        False
+        >>> Bin(4, 5) < Bin(3, 5)
+        False
+        >>> Bin(3, 5) < 4
+        True
+        >>> Bin(3, 5) < 3
+        False
+        >>> Bin(4, 5) < 3
+        False
+        """
+        other = self._coerce_same_n(other)
+        return self.int < other.int
 
-    # def __lt__(self, other):
-    #     return self.int < other
+    def __le__(self, other):
+        other = self._coerce_same_n(other)
+        return self.int <= other.int
 
-    # def __le__(self, other):
-    #     return self.int <= other
+    def __gt__(self, other):
+        other = self._coerce_same_n(other)
+        return self.int > other.int
 
-    # def __gt__(self, other):
-    #     return self.int > other
-
-    # def __ge__(self, other):
-    #     return self.int >= other
+    def __ge__(self, other):
+        other = self._coerce_same_n(other)
+        return self.int >= other.int
 
     def __bool__(self):
         """
@@ -463,7 +514,15 @@ class Bin:
         return self.int != 0
 
     def __hash__(self):
-        return hash((self.n, self.int))
+        """
+        Hash matches that of underlying int to allow reusing Bin as dict keys
+
+        >>> d = {10: 123}; d[Bin(10, 5)] == d[Bin(10, 100)] == 123
+        True
+        >>> hash(100) == hash(Bin(100, 7)) == hash(Bin(100, 10))
+        True
+        """
+        return hash(self.int)
 
     def is_prec(self, other):
         return self.int & other == self.int and self.int != other
@@ -584,6 +643,37 @@ class Bin:
         y = self.int ^ self.mask
         return self._new(y, n=self.n)
 
+    def __neg__(self):
+        """
+        >>> -Bin(1, 4)
+        Bin(0b1111, n=4)
+        >>> -Bin(0, 4)
+        Bin(0b0000, n=4)
+        """
+        mod = 1 << self.n
+        v = mod - self.int
+        if v >= mod:
+            v -= mod
+        return self._new(v, n=self.n)
+
+    def __add__(self, other):
+        """
+        >>> Bin(31, 5) + 2
+        Bin(0b00001, n=5)
+        >>> Bin(31, 5) + Bin(1, 5)
+        Bin(0b00000, n=5)
+        >>> Bin(31, 5) + Bin(1, 6)
+        Bin(0b100000, n=6)
+        """
+        if isinstance(other, Bin):
+            n = max(self.n, other.n)
+            other = other.int
+        else:
+            n = self.n
+        mod = 1 << n
+        v = int(self.int + other) % mod
+        return self._new(v, n=n)
+
     def scalar_bin(self, other):
         """Dot product in GF(2).
 
